package aitoa.examples.jssp;

import java.util.concurrent.ThreadLocalRandom;

import org.junit.Assert;
import org.junit.Test;

import aitoa.structure.IObjectiveFunction;
import aitoa.structure.IObjectiveFunctionTest;

/** A Test for the JSSP MakeSpan Objective Function */
public class TestJSSPMakespanObjective
    extends IObjectiveFunctionTest<JSSPCandidateSolution> {

  /** the example instance */
  public static final JSSPInstance PROBLEM =
      new JSSPInstance("swv17"); //$NON-NLS-1$

  /** the example instance */
  public static final JSSPMakespanObjectiveFunction F =
      new JSSPMakespanObjectiveFunction(
          TestJSSPMakespanObjective.PROBLEM);

  /** the internal mapping */
  private static final JSSPRepresentationMapping MAPPING =
      new JSSPRepresentationMapping(
          TestJSSPMakespanObjective.PROBLEM);

  /** {@inheritDoc} */
  @Override
  protected IObjectiveFunction<JSSPCandidateSolution>
      getInstance() {
    return TestJSSPMakespanObjective.F;
  }

  /** {@inheritDoc} */
  @Override
  protected JSSPCandidateSolution createValid() {
    final JSSPCandidateSolution y = new JSSPCandidateSolution(
        TestJSSPMakespanObjective.PROBLEM.m,
        TestJSSPMakespanObjective.PROBLEM.n);
    TestJSSPMakespanObjective.MAPPING
        .map(
            ThreadLocalRandom.current(), JSSPTestUtils
                .createValidX(TestJSSPMakespanObjective.PROBLEM),
            y);
    return y;
  }

  /** the example instance */
  public static final JSSPInstance INSTANCE =
      new JSSPInstance("demo"); //$NON-NLS-1$

  /** test the makespan */
  @SuppressWarnings("static-method")
  @Test(timeout = 3600000)
  public void testObjective() {
    final JSSPMakespanObjectiveFunction f =
        new JSSPMakespanObjectiveFunction(
            TestJSSPMakespanObjective.INSTANCE);
    JSSPCandidateSolution x;

    x = new JSSPCandidateSolution(0, 0);
    Assert.assertEquals(0d, f.evaluate(x), 0d);

    x = new JSSPCandidateSolution(1, 1);
    Assert.assertEquals(0d, f.evaluate(x), 0d);
    x.schedule[0][2] = 5;
    Assert.assertEquals(5d, f.evaluate(x), 0d);

    x = new JSSPCandidateSolution(2, 1);
    Assert.assertEquals(0d, f.evaluate(x), 0d);
    x.schedule[0][2] = 5;
    x.schedule[1][2] = 4;
    Assert.assertEquals(5d, f.evaluate(x), 0d);
    x.schedule[1][2] = 5;
    x.schedule[0][2] = 4;
    Assert.assertEquals(5d, f.evaluate(x), 0d);
    x.schedule[0][2] = 3;
    x.schedule[1][2] = 3;
    Assert.assertEquals(3d, f.evaluate(x), 0d);

    x = new JSSPCandidateSolution(2, 3);
    Assert.assertEquals(0d, f.evaluate(x), 0d);
    x.schedule[0][8] = 5;
    x.schedule[1][8] = 4;
    Assert.assertEquals(5d, f.evaluate(x), 0d);
    x.schedule[1][8] = 5;
    x.schedule[0][8] = 4;
    Assert.assertEquals(5d, f.evaluate(x), 0d);
    x.schedule[0][8] = 3;
    x.schedule[1][8] = 3;
    Assert.assertEquals(3d, f.evaluate(x), 0d);

    x = new JSSPCandidateSolution(4, 3);
    Assert.assertEquals(0d, f.evaluate(x), 0d);
    x.schedule[0][8] = 5;
    x.schedule[1][8] = 4;
    x.schedule[2][8] = 2;
    x.schedule[3][8] = 3;
    Assert.assertEquals(5d, f.evaluate(x), 0d);
    x.schedule[0][8] = 4;
    x.schedule[1][8] = 5;
    x.schedule[2][8] = 2;
    x.schedule[3][8] = 3;
    Assert.assertEquals(5d, f.evaluate(x), 0d);
    x.schedule[0][8] = 2;
    x.schedule[1][8] = 4;
    x.schedule[2][8] = 5;
    x.schedule[3][8] = 3;
    Assert.assertEquals(5d, f.evaluate(x), 0d);
  }

  /** test the makespan */
  @SuppressWarnings("static-method")
  @Test(timeout = 3600000)
  public void testLowerBounds() {
    Assert
        .assertEquals(1005,
            (int) (new JSSPMakespanObjectiveFunction(
                new JSSPInstance(
                    new int[][] {
                        { 6, 94, 12, 66, 4, 10, 7, 53, 3, 26, 2,
                            15, 10, 65, 11, 82, 8, 10, 14, 27, 9,
                            93, 13, 92, 5, 96, 0, 70, 1, 83 },
                        { 4, 74, 5, 31, 7, 88, 14, 51, 13, 57, 8,
                            78, 11, 8, 9, 7, 6, 91, 10, 79, 0,
                            18, 3, 51, 12, 18, 1, 99, 2, 33 },
                        { 1, 4, 8, 82, 9, 40, 12, 86, 6, 50, 11,
                            54, 13, 21, 5, 6, 0, 54, 2, 68, 7,
                            82, 10, 20, 4, 39, 3, 35, 14, 68 },
                        { 5, 73, 2, 23, 9, 30, 6, 30, 10, 53, 0,
                            94, 13, 58, 4, 93, 7, 32, 14, 91, 11,
                            30, 8, 56, 12, 27, 1, 92, 3, 9 },
                        { 7, 78, 8, 23, 6, 21, 10, 60, 4, 36, 9,
                            29, 2, 95, 14, 99, 12, 79, 5, 76, 1,
                            93, 13, 42, 11, 52, 0, 42, 3, 96 },
                        { 5, 29, 3, 61, 12, 88, 13, 70, 11, 16,
                            4, 31, 14, 65, 7, 83, 2, 78, 1, 26,
                            10, 50, 0, 87, 9, 62, 6, 14, 8, 30 },
                        { 12, 18, 3, 75, 7, 20, 8, 4, 14, 91, 6,
                            68, 1, 19, 11, 54, 4, 85, 5, 73, 2,
                            43, 10, 24, 0, 37, 13, 87, 9, 66 },
                        { 11, 32, 5, 52, 0, 9, 7, 49, 12, 61, 13,
                            35, 14, 99, 1, 62, 2, 6, 8, 62, 4, 7,
                            3, 80, 9, 3, 6, 57, 10, 7 },
                        { 10, 85, 11, 30, 6, 96, 14, 91, 0, 13,
                            1, 87, 2, 82, 5, 83, 12, 78, 4, 56,
                            8, 85, 7, 8, 9, 66, 13, 88, 3, 15 },
                        { 6, 5, 11, 59, 9, 30, 2, 60, 8, 41, 0,
                            17, 13, 66, 3, 89, 10, 78, 7, 88, 1,
                            69, 12, 45, 14, 82, 4, 6, 5, 13 },
                        { 4, 90, 7, 27, 13, 1, 0, 8, 5, 91, 12,
                            80, 6, 89, 8, 49, 14, 32, 10, 28, 3,
                            90, 1, 93, 11, 6, 9, 35, 2, 73 },
                        { 2, 47, 14, 43, 0, 75, 12, 8, 6, 51, 10,
                            3, 7, 84, 5, 34, 8, 28, 9, 60, 13,
                            69, 1, 45, 3, 67, 11, 58, 4, 87 },
                        { 5, 65, 8, 62, 10, 97, 2, 20, 3, 31, 6,
                            33, 9, 33, 0, 77, 13, 50, 4, 80, 1,
                            48, 11, 90, 12, 75, 7, 96, 14, 44 },
                        { 8, 28, 14, 21, 4, 51, 13, 75, 5, 17, 6,
                            89, 9, 59, 1, 56, 12, 63, 7, 18, 11,
                            17, 10, 30, 3, 16, 2, 7, 0, 35 },
                        { 10, 57, 8, 16, 12, 42, 6, 34, 4, 37, 1,
                            26, 13, 68, 14, 73, 11, 5, 0, 8, 7,
                            12, 3, 87, 2, 83, 9, 20, 5, 97 } },
                    "ta01")).lowerBound())); //$NON-NLS-1$

    Assert
        .assertEquals(1254,
            (int) (new JSSPMakespanObjectiveFunction(
                new JSSPInstance(
                    new int[][] {
                        { 3, 25, 11, 75, 14, 75, 1, 76, 10, 38,
                            2, 62, 4, 38, 7, 59, 0, 14, 12, 13,
                            5, 46, 9, 31, 6, 57, 13, 92, 8, 3 },
                        { 5, 67, 0, 5, 3, 11, 8, 11, 4, 40, 1,
                            34, 12, 77, 14, 42, 6, 35, 7, 96, 10,
                            22, 2, 55, 9, 21, 13, 29, 11, 16 },
                        { 2, 22, 3, 98, 14, 8, 0, 35, 9, 59, 12,
                            31, 5, 13, 4, 46, 7, 52, 10, 22, 8,
                            18, 11, 19, 13, 64, 1, 29, 6, 70 },
                        { 8, 99, 10, 42, 1, 2, 13, 35, 3, 11, 4,
                            92, 14, 88, 9, 97, 2, 21, 5, 56, 11,
                            17, 7, 43, 0, 27, 6, 19, 12, 23 },
                        { 14, 50, 8, 5, 1, 59, 2, 71, 10, 47, 9,
                            39, 12, 82, 4, 35, 6, 12, 5, 2, 0,
                            39, 13, 42, 3, 52, 11, 65, 7, 35 },
                        { 3, 48, 10, 57, 1, 5, 5, 2, 6, 60, 0,
                            64, 8, 86, 7, 3, 11, 51, 13, 26, 2,
                            34, 14, 39, 12, 45, 9, 63, 4, 54 },
                        { 2, 40, 10, 43, 1, 50, 12, 71, 8, 46, 0,
                            99, 7, 67, 6, 34, 14, 6, 13, 95, 4,
                            67, 3, 54, 5, 29, 9, 30, 11, 60 },
                        { 1, 59, 0, 3, 2, 85, 4, 6, 7, 46, 13,
                            49, 11, 5, 3, 82, 12, 18, 5, 71, 6,
                            48, 14, 79, 9, 62, 8, 65, 10, 76 },
                        { 4, 65, 5, 55, 9, 81, 10, 15, 7, 32, 6,
                            52, 2, 97, 1, 69, 12, 82, 3, 89, 13,
                            69, 0, 87, 8, 22, 14, 71, 11, 63 },
                        { 1, 70, 4, 74, 3, 52, 10, 94, 14, 14, 0,
                            81, 6, 24, 13, 14, 11, 32, 8, 39, 5,
                            67, 12, 59, 7, 18, 9, 77, 2, 50 },
                        { 3, 18, 10, 6, 1, 96, 0, 53, 9, 35, 8,
                            99, 14, 39, 6, 18, 4, 14, 7, 90, 2,
                            64, 12, 81, 5, 89, 11, 48, 13, 80 },
                        { 2, 44, 7, 75, 6, 12, 8, 13, 3, 74, 5,
                            59, 14, 71, 4, 75, 1, 30, 0, 93, 9,
                            26, 10, 30, 13, 84, 11, 91, 12, 93 },
                        { 0, 39, 7, 56, 14, 13, 8, 29, 12, 55,
                            10, 69, 9, 26, 3, 7, 6, 55, 1, 48, 4,
                            22, 2, 46, 11, 50, 13, 96, 5, 17 },
                        { 12, 57, 3, 14, 9, 8, 4, 13, 1, 95, 0,
                            53, 10, 78, 6, 24, 5, 92, 2, 90, 14,
                            68, 13, 87, 7, 43, 8, 75, 11, 94 },
                        { 3, 93, 14, 92, 6, 18, 5, 28, 13, 27, 9,
                            40, 1, 56, 0, 83, 12, 51, 7, 15, 2,
                            97, 4, 48, 10, 53, 8, 78, 11, 39 },
                        { 5, 47, 14, 34, 6, 42, 12, 28, 8, 11, 2,
                            11, 4, 30, 9, 14, 11, 10, 13, 4, 3,
                            20, 1, 92, 7, 19, 0, 59, 10, 28 },
                        { 3, 69, 7, 82, 10, 64, 14, 40, 0, 27, 8,
                            82, 1, 27, 11, 43, 5, 56, 13, 17, 4,
                            18, 12, 20, 6, 98, 9, 43, 2, 68 },
                        { 10, 84, 8, 26, 2, 87, 11, 61, 13, 95,
                            6, 23, 14, 88, 3, 89, 9, 49, 7, 84,
                            4, 12, 5, 51, 12, 3, 0, 44, 1, 20 },
                        { 3, 43, 2, 54, 12, 18, 13, 72, 1, 70, 6,
                            28, 14, 20, 5, 22, 4, 59, 8, 36, 9,
                            85, 11, 13, 0, 73, 10, 29, 7, 45 },
                        { 11, 7, 14, 97, 5, 4, 6, 22, 10, 74, 9,
                            45, 13, 62, 1, 95, 4, 66, 8, 14, 0,
                            40, 3, 23, 12, 79, 2, 34, 7, 8 } },
                    "ta11")).lowerBound())); //$NON-NLS-1$

    Assert.assertEquals(5183,
        (int) (new JSSPMakespanObjectiveFunction(
            new JSSPInstance(new int[][] {
                { 9, 54, 12, 87, 4, 61, 7, 35, 16, 5, 0, 48, 1,
                    33, 18, 21, 17, 65, 5, 83, 14, 78, 11, 14, 8,
                    70, 10, 25, 3, 36, 15, 56, 13, 87, 2, 32, 19,
                    96, 6, 62 },
                { 17, 68, 13, 15, 19, 22, 6, 53, 10, 33, 11, 61,
                    1, 73, 7, 63, 16, 96, 18, 61, 14, 88, 15, 86,
                    5, 53, 3, 2, 9, 29, 2, 14, 0, 49, 4, 14, 8,
                    22, 12, 66 },
                { 17, 68, 10, 70, 3, 84, 4, 19, 19, 32, 7, 58,
                    15, 87, 0, 82, 16, 7, 12, 47, 14, 68, 18, 71,
                    8, 3, 11, 93, 6, 24, 2, 31, 5, 19, 1, 56, 9,
                    88, 13, 71 },
                { 9, 58, 19, 72, 3, 51, 10, 42, 1, 31, 5, 63, 16,
                    49, 14, 83, 7, 28, 17, 92, 18, 83, 13, 7, 4,
                    7, 0, 30, 11, 14, 15, 27, 8, 58, 12, 32, 2,
                    16, 6, 67 },
                { 13, 36, 16, 51, 4, 65, 15, 40, 9, 79, 10, 39,
                    0, 33, 2, 77, 3, 84, 11, 68, 17, 71, 5, 54,
                    19, 40, 7, 94, 18, 56, 6, 94, 1, 55, 12, 10,
                    14, 13, 8, 13 },
                { 16, 51, 7, 27, 9, 31, 6, 65, 13, 26, 5, 62, 18,
                    88, 11, 69, 17, 47, 19, 36, 14, 3, 15, 90,
                    10, 90, 0, 31, 4, 24, 12, 35, 2, 24, 1, 68,
                    3, 18, 8, 70 },
                { 14, 8, 5, 6, 1, 93, 10, 60, 9, 68, 13, 32, 16,
                    95, 17, 17, 2, 12, 18, 79, 19, 78, 12, 26,
                    15, 15, 6, 83, 0, 8, 3, 31, 7, 5, 4, 6, 11,
                    36, 8, 75 },
                { 1, 41, 11, 95, 10, 4, 3, 98, 17, 22, 9, 64, 14,
                    41, 18, 56, 0, 23, 4, 72, 15, 10, 16, 35, 6,
                    55, 12, 17, 7, 10, 5, 32, 13, 76, 8, 49, 19,
                    95, 2, 14 },
                { 2, 96, 17, 23, 8, 44, 19, 19, 10, 90, 13, 6,
                    15, 67, 12, 37, 16, 64, 1, 80, 18, 98, 6, 66,
                    11, 74, 0, 65, 14, 3, 3, 15, 9, 50, 7, 98, 5,
                    46, 4, 80 },
                { 3, 1, 17, 65, 11, 36, 0, 35, 5, 95, 15, 99, 13,
                    82, 12, 46, 16, 26, 10, 35, 2, 96, 6, 6, 4,
                    28, 9, 92, 19, 12, 8, 42, 1, 47, 7, 78, 18,
                    10, 14, 76 },
                { 2, 78, 8, 99, 14, 90, 5, 46, 15, 71, 13, 76,
                    10, 45, 19, 96, 12, 58, 9, 3, 4, 70, 0, 80,
                    6, 30, 3, 85, 18, 93, 7, 16, 16, 26, 11, 79,
                    1, 44, 17, 21 },
                { 1, 82, 13, 16, 8, 56, 3, 35, 7, 81, 14, 97, 9,
                    5, 6, 68, 10, 60, 15, 33, 19, 57, 11, 42, 4,
                    72, 16, 77, 2, 53, 0, 26, 18, 66, 17, 81, 5,
                    84, 12, 84 },
                { 5, 29, 0, 6, 7, 38, 1, 96, 4, 85, 8, 36, 10,
                    71, 3, 5, 13, 53, 9, 11, 11, 87, 17, 99, 12,
                    42, 14, 69, 15, 74, 6, 34, 2, 25, 19, 10, 16,
                    25, 18, 32 },
                { 7, 40, 19, 73, 5, 80, 6, 70, 9, 7, 14, 35, 8,
                    35, 4, 20, 3, 68, 15, 29, 1, 7, 10, 12, 11,
                    70, 12, 47, 2, 46, 16, 78, 0, 28, 13, 48, 17,
                    50, 18, 22 },
                { 2, 94, 18, 79, 0, 1, 13, 39, 6, 33, 16, 94, 5,
                    69, 19, 19, 14, 29, 9, 33, 7, 72, 3, 48, 1,
                    88, 10, 9, 17, 1, 8, 99, 4, 20, 11, 29, 12,
                    83, 15, 44 },
                { 12, 4, 11, 18, 1, 9, 17, 82, 4, 59, 13, 64, 16,
                    56, 18, 78, 6, 23, 7, 61, 8, 80, 10, 91, 2,
                    38, 14, 89, 5, 85, 3, 23, 9, 48, 19, 90, 0,
                    29, 15, 97 },
                { 11, 10, 8, 9, 10, 82, 17, 28, 3, 48, 14, 66, 1,
                    22, 9, 32, 13, 34, 5, 43, 19, 64, 4, 43, 18,
                    24, 6, 88, 16, 44, 12, 15, 7, 28, 0, 54, 2,
                    94, 15, 80 },
                { 2, 50, 7, 54, 15, 27, 0, 69, 17, 34, 18, 5, 10,
                    22, 3, 29, 16, 47, 11, 24, 19, 52, 13, 26, 9,
                    51, 14, 58, 12, 50, 5, 84, 8, 15, 1, 93, 4,
                    2, 6, 16 },
                { 16, 71, 12, 25, 15, 2, 11, 75, 5, 62, 3, 79, 2,
                    35, 8, 87, 0, 19, 6, 50, 10, 33, 7, 79, 9,
                    63, 18, 9, 17, 24, 19, 39, 4, 2, 14, 20, 13,
                    84, 1, 53 },
                { 1, 78, 14, 11, 19, 42, 0, 60, 10, 76, 16, 57,
                    15, 29, 7, 50, 17, 79, 18, 81, 3, 19, 4, 24,
                    13, 91, 8, 57, 5, 80, 9, 74, 6, 99, 11, 4, 2,
                    62, 12, 2 },
                { 12, 4, 9, 65, 2, 42, 15, 97, 14, 56, 8, 92, 17,
                    49, 6, 83, 1, 18, 11, 86, 10, 48, 5, 24, 16,
                    42, 13, 99, 4, 87, 3, 59, 0, 22, 7, 34, 18,
                    63, 19, 45 },
                { 11, 54, 17, 64, 18, 80, 12, 51, 19, 75, 14, 42,
                    8, 60, 10, 33, 15, 6, 7, 90, 2, 32, 1, 77, 4,
                    11, 16, 63, 13, 40, 9, 30, 0, 37, 6, 62, 3,
                    94, 5, 8 },
                { 18, 6, 12, 77, 17, 76, 8, 23, 5, 61, 10, 90, 6,
                    6, 9, 85, 13, 45, 0, 86, 2, 10, 15, 71, 7,
                    13, 4, 68, 16, 16, 1, 33, 14, 95, 11, 52, 3,
                    88, 19, 39 },
                { 9, 47, 12, 30, 8, 67, 18, 99, 5, 52, 2, 29, 15,
                    23, 11, 8, 0, 77, 6, 80, 1, 46, 4, 54, 17,
                    64, 3, 45, 13, 17, 7, 12, 14, 35, 19, 3, 10,
                    81, 16, 15 },
                { 18, 24, 16, 50, 17, 47, 11, 88, 6, 68, 8, 42,
                    1, 2, 7, 24, 12, 2, 10, 63, 3, 88, 9, 72, 14,
                    28, 0, 47, 19, 31, 2, 62, 5, 59, 4, 6, 15, 9,
                    13, 60 },
                { 17, 68, 15, 82, 6, 22, 4, 44, 7, 55, 1, 25, 0,
                    45, 13, 75, 9, 13, 19, 84, 18, 17, 5, 67, 3,
                    21, 10, 6, 2, 49, 16, 67, 11, 7, 8, 74, 14,
                    29, 12, 20 },
                { 3, 73, 15, 2, 7, 26, 8, 15, 10, 29, 9, 44, 2,
                    26, 17, 99, 19, 86, 14, 59, 16, 13, 5, 90, 1,
                    13, 18, 17, 12, 68, 11, 88, 0, 42, 4, 61, 13,
                    65, 6, 72 },
                { 3, 3, 17, 1, 12, 40, 9, 33, 4, 29, 10, 42, 15,
                    13, 13, 15, 0, 78, 11, 37, 8, 30, 2, 92, 1,
                    49, 5, 91, 19, 42, 7, 74, 16, 53, 18, 59, 6,
                    56, 14, 32 },
                { 19, 31, 16, 84, 18, 29, 13, 75, 5, 68, 11, 12,
                    8, 29, 9, 51, 0, 76, 2, 98, 14, 33, 17, 99,
                    10, 28, 4, 17, 6, 89, 1, 4, 3, 21, 12, 84, 7,
                    90, 15, 4 },
                { 18, 5, 8, 69, 2, 19, 6, 45, 12, 35, 17, 89, 19,
                    40, 10, 38, 9, 30, 15, 52, 5, 73, 1, 7, 3,
                    45, 7, 60, 14, 43, 13, 55, 11, 10, 4, 97, 0,
                    93, 16, 63 },
                { 12, 76, 13, 61, 1, 21, 3, 41, 18, 74, 17, 51,
                    15, 36, 6, 49, 5, 26, 9, 31, 11, 67, 16, 69,
                    14, 6, 10, 54, 0, 65, 8, 7, 4, 70, 7, 45, 2,
                    32, 19, 25 },
                { 7, 1, 8, 69, 13, 28, 0, 25, 16, 25, 12, 24, 15,
                    74, 17, 48, 6, 4, 4, 12, 11, 80, 18, 28, 10,
                    75, 9, 82, 2, 34, 14, 99, 5, 19, 19, 59, 1,
                    10, 3, 11 },
                { 11, 46, 17, 81, 19, 64, 7, 28, 6, 89, 12, 71,
                    16, 29, 9, 27, 13, 81, 0, 34, 2, 78, 15, 7,
                    10, 51, 5, 25, 4, 47, 1, 14, 18, 32, 14, 47,
                    3, 90, 8, 31 },
                { 6, 4, 8, 40, 9, 32, 17, 52, 16, 37, 4, 35, 10,
                    85, 14, 26, 18, 45, 2, 58, 7, 42, 19, 75, 5,
                    13, 0, 15, 3, 86, 11, 74, 13, 98, 1, 19, 15,
                    45, 12, 60 },
                { 16, 44, 8, 65, 3, 5, 4, 30, 13, 51, 1, 42, 2,
                    88, 9, 79, 10, 47, 7, 46, 12, 62, 6, 50, 17,
                    58, 5, 82, 0, 51, 18, 89, 11, 89, 19, 63, 15,
                    59, 14, 84 },
                { 7, 50, 11, 61, 2, 36, 15, 14, 16, 65, 4, 86,
                    18, 87, 0, 28, 17, 50, 13, 8, 19, 99, 10, 73,
                    9, 96, 12, 94, 1, 87, 6, 68, 14, 96, 8, 69,
                    3, 19, 5, 33 },
                { 12, 81, 6, 81, 0, 29, 17, 54, 5, 24, 10, 55,
                    13, 5, 8, 36, 1, 48, 16, 41, 4, 34, 7, 33,
                    11, 48, 19, 37, 18, 32, 2, 34, 9, 52, 14, 27,
                    3, 80, 15, 14 },
                { 2, 36, 8, 31, 7, 7, 10, 88, 17, 99, 6, 55, 13,
                    11, 3, 65, 19, 47, 18, 18, 9, 46, 12, 39, 11,
                    78, 14, 46, 1, 81, 16, 31, 5, 80, 4, 35, 15,
                    92, 0, 51 },
                { 10, 65, 3, 46, 1, 31, 15, 3, 14, 21, 11, 55,
                    13, 11, 2, 5, 16, 40, 4, 52, 8, 99, 18, 11,
                    12, 36, 9, 83, 5, 92, 7, 5, 19, 19, 0, 58,
                    17, 47, 6, 34 },
                { 8, 50, 14, 56, 19, 57, 2, 27, 18, 20, 17, 26,
                    16, 71, 5, 59, 6, 14, 13, 46, 10, 22, 3, 66,
                    7, 20, 0, 33, 4, 39, 12, 33, 9, 86, 1, 30,
                    15, 37, 11, 77 },
                { 9, 1, 1, 51, 19, 77, 6, 72, 4, 87, 17, 50, 13,
                    94, 7, 84, 15, 8, 8, 73, 0, 13, 2, 12, 10,
                    97, 16, 86, 14, 63, 18, 44, 5, 14, 3, 67, 12,
                    19, 11, 52 },
                { 8, 22, 7, 5, 4, 51, 14, 18, 18, 23, 16, 97, 19,
                    91, 15, 86, 2, 13, 10, 27, 6, 68, 12, 44, 9,
                    93, 5, 82, 0, 17, 1, 48, 13, 98, 3, 30, 17,
                    86, 11, 92 },
                { 2, 5, 9, 92, 18, 10, 7, 63, 16, 41, 13, 26, 14,
                    14, 15, 30, 10, 3, 17, 9, 11, 99, 3, 54, 5,
                    14, 0, 98, 8, 23, 12, 11, 4, 66, 19, 48, 1,
                    25, 6, 79 },
                { 6, 82, 2, 50, 16, 86, 10, 19, 18, 70, 0, 76,
                    17, 99, 19, 34, 3, 53, 5, 23, 7, 89, 14, 82,
                    9, 19, 12, 55, 15, 15, 8, 23, 4, 99, 13, 63,
                    1, 1, 11, 75 },
                { 10, 43, 0, 25, 12, 40, 1, 18, 6, 43, 14, 86,
                    18, 75, 11, 24, 7, 82, 16, 24, 9, 71, 8, 89,
                    4, 67, 15, 28, 13, 96, 19, 20, 5, 14, 3, 85,
                    2, 59, 17, 38 },
                { 14, 28, 5, 8, 4, 99, 19, 8, 9, 53, 8, 51, 3,
                    92, 12, 61, 1, 79, 0, 73, 15, 80, 18, 24, 17,
                    55, 13, 73, 10, 55, 6, 9, 11, 66, 7, 22, 16,
                    95, 2, 60 },
                { 2, 14, 13, 89, 17, 24, 11, 37, 4, 30, 9, 88,
                    15, 1, 1, 15, 7, 14, 0, 36, 5, 14, 12, 62,
                    16, 3, 14, 27, 8, 44, 10, 50, 3, 63, 6, 26,
                    19, 91, 18, 85 },
                { 2, 24, 5, 74, 14, 42, 10, 43, 1, 58, 7, 29, 3,
                    58, 18, 54, 19, 76, 6, 22, 4, 42, 8, 80, 17,
                    45, 12, 28, 15, 32, 0, 59, 11, 1, 13, 81, 16,
                    48, 9, 15 },
                { 4, 90, 13, 20, 10, 88, 15, 84, 14, 39, 12, 6,
                    18, 12, 11, 14, 1, 19, 5, 44, 17, 10, 9, 26,
                    2, 6, 16, 75, 0, 24, 8, 38, 19, 53, 6, 37, 3,
                    69, 7, 46 },
                { 3, 54, 17, 78, 4, 20, 13, 66, 8, 2, 15, 52, 19,
                    47, 7, 84, 6, 28, 11, 11, 12, 66, 14, 45, 1,
                    8, 10, 22, 9, 14, 0, 31, 5, 88, 18, 73, 16,
                    21, 2, 70 },
                { 11, 36, 8, 29, 0, 65, 13, 32, 5, 79, 7, 2, 10,
                    42, 9, 95, 2, 33, 4, 5, 1, 62, 12, 47, 17,
                    20, 6, 2, 18, 2, 15, 36, 14, 22, 19, 96, 3,
                    61, 16, 85 },
                { 3, 27, 11, 22, 15, 60, 8, 87, 10, 30, 6, 64, 5,
                    75, 18, 26, 2, 25, 9, 76, 19, 28, 16, 53, 14,
                    65, 17, 59, 12, 92, 4, 54, 7, 92, 1, 28, 13,
                    20, 0, 48 },
                { 10, 57, 18, 13, 3, 93, 8, 47, 2, 40, 12, 81,
                    11, 89, 13, 52, 6, 58, 14, 20, 19, 19, 1, 37,
                    16, 69, 17, 1, 0, 13, 7, 60, 4, 83, 15, 35,
                    9, 64, 5, 73 },
                { 3, 63, 14, 86, 0, 36, 4, 85, 5, 91, 7, 12, 13,
                    46, 6, 52, 15, 70, 17, 25, 12, 50, 10, 41, 1,
                    4, 9, 9, 11, 36, 2, 99, 18, 57, 8, 91, 16,
                    96, 19, 43 },
                { 2, 15, 8, 84, 7, 73, 9, 93, 19, 51, 16, 14, 17,
                    25, 15, 75, 6, 39, 3, 14, 4, 43, 13, 73, 10,
                    82, 12, 2, 0, 72, 5, 66, 14, 30, 18, 64, 1,
                    19, 11, 63 },
                { 18, 38, 12, 74, 11, 59, 7, 69, 17, 60, 8, 99,
                    13, 14, 15, 48, 9, 9, 14, 50, 16, 83, 5, 85,
                    2, 74, 10, 10, 19, 95, 3, 10, 1, 80, 0, 92,
                    6, 92, 4, 39 },
                { 0, 48, 7, 75, 2, 18, 16, 65, 17, 11, 4, 32, 13,
                    61, 1, 46, 9, 22, 10, 1, 11, 38, 14, 33, 15,
                    78, 3, 39, 5, 67, 6, 79, 12, 66, 8, 99, 19,
                    22, 18, 66 },
                { 3, 86, 13, 95, 18, 30, 9, 19, 17, 90, 12, 65,
                    14, 79, 19, 23, 7, 69, 10, 81, 0, 30, 11, 9,
                    5, 99, 8, 86, 2, 66, 4, 62, 16, 32, 1, 97, 6,
                    25, 15, 37 },
                { 7, 42, 18, 67, 10, 84, 5, 55, 8, 76, 13, 48, 2,
                    95, 16, 59, 3, 69, 14, 53, 1, 65, 4, 30, 19,
                    94, 9, 85, 15, 15, 0, 96, 11, 68, 17, 1, 12,
                    30, 6, 94 },
                { 6, 28, 12, 76, 8, 2, 0, 86, 4, 58, 16, 40, 5,
                    14, 19, 32, 11, 11, 2, 13, 14, 1, 13, 56, 7,
                    92, 18, 13, 3, 85, 10, 15, 9, 74, 1, 51, 17,
                    4, 15, 66 },
                { 8, 52, 17, 73, 15, 57, 10, 13, 6, 52, 3, 49, 5,
                    38, 1, 14, 9, 87, 2, 63, 13, 78, 11, 40, 14,
                    13, 7, 23, 12, 42, 18, 99, 0, 67, 4, 28, 19,
                    65, 16, 92 },
                { 2, 90, 7, 59, 17, 86, 10, 30, 5, 34, 12, 79,
                    13, 77, 18, 9, 0, 84, 15, 73, 4, 9, 1, 44, 9,
                    43, 8, 59, 6, 6, 11, 90, 19, 3, 3, 19, 14, 4,
                    16, 33 },
                { 15, 34, 5, 18, 4, 25, 19, 51, 8, 21, 13, 93, 3,
                    22, 0, 56, 14, 80, 1, 77, 12, 67, 11, 89, 10,
                    15, 17, 60, 9, 15, 2, 80, 18, 23, 7, 90, 6,
                    86, 16, 17 },
                { 10, 21, 16, 61, 19, 25, 6, 91, 2, 67, 5, 84, 0,
                    15, 1, 59, 9, 35, 4, 5, 8, 38, 12, 9, 3, 54,
                    17, 58, 14, 36, 7, 36, 13, 69, 15, 7, 11, 22,
                    18, 5 },
                { 17, 3, 4, 73, 3, 36, 11, 75, 6, 95, 12, 71, 18,
                    10, 14, 7, 9, 13, 19, 33, 7, 39, 15, 30, 10,
                    82, 0, 76, 5, 68, 13, 37, 1, 62, 2, 44, 16,
                    6, 8, 51 },
                { 19, 87, 1, 31, 4, 28, 6, 68, 14, 7, 2, 85, 3,
                    57, 8, 82, 16, 98, 11, 16, 18, 12, 13, 28,
                    10, 5, 15, 56, 7, 78, 9, 7, 5, 18, 12, 7, 0,
                    9, 17, 15 },
                { 16, 8, 17, 2, 11, 50, 1, 51, 8, 97, 6, 86, 7,
                    72, 13, 92, 18, 29, 4, 58, 3, 10, 0, 13, 10,
                    25, 9, 42, 2, 11, 14, 46, 19, 74, 5, 86, 12,
                    20, 15, 31 },
                { 8, 43, 3, 4, 1, 31, 14, 98, 13, 38, 7, 13, 19,
                    9, 10, 87, 0, 68, 17, 54, 12, 75, 2, 37, 4,
                    11, 16, 58, 18, 46, 6, 49, 5, 6, 9, 15, 15,
                    28, 11, 86 },
                { 13, 80, 16, 90, 4, 47, 17, 26, 7, 9, 19, 10, 6,
                    18, 3, 95, 2, 19, 12, 52, 11, 17, 10, 55, 1,
                    74, 14, 24, 9, 85, 15, 40, 0, 62, 8, 60, 5,
                    96, 18, 96 },
                { 6, 30, 17, 75, 7, 59, 14, 22, 5, 4, 4, 52, 3,
                    97, 10, 39, 1, 49, 12, 36, 8, 91, 15, 79, 2,
                    17, 18, 38, 11, 4, 9, 57, 16, 44, 13, 46, 19,
                    17, 0, 66 },
                { 11, 3, 6, 70, 10, 95, 4, 22, 8, 72, 18, 39, 0,
                    96, 16, 94, 2, 47, 17, 35, 5, 37, 14, 59, 1,
                    45, 15, 87, 13, 99, 7, 33, 3, 48, 9, 4, 19,
                    92, 12, 68 },
                { 7, 18, 17, 74, 15, 88, 1, 60, 12, 8, 9, 45, 5,
                    1, 14, 83, 0, 71, 2, 78, 4, 71, 11, 52, 13,
                    36, 18, 18, 16, 12, 3, 93, 19, 72, 6, 37, 10,
                    32, 8, 28 },
                { 11, 46, 16, 80, 10, 12, 17, 43, 18, 76, 5, 83,
                    13, 74, 19, 44, 7, 97, 14, 46, 12, 30, 15,
                    56, 4, 79, 9, 34, 2, 56, 3, 49, 1, 2, 6, 63,
                    8, 41, 0, 87 },
                { 2, 53, 13, 35, 19, 98, 4, 19, 6, 22, 7, 31, 10,
                    89, 18, 94, 14, 27, 17, 92, 12, 80, 16, 60,
                    11, 22, 8, 31, 15, 23, 5, 5, 9, 40, 0, 5, 1,
                    90, 3, 13 },
                { 16, 32, 11, 43, 3, 75, 10, 15, 18, 37, 14, 5,
                    1, 64, 7, 4, 6, 30, 17, 29, 19, 14, 15, 35,
                    2, 85, 0, 75, 8, 45, 13, 45, 4, 2, 12, 7, 9,
                    29, 5, 68 },
                { 5, 95, 13, 11, 0, 5, 1, 20, 3, 49, 6, 16, 17,
                    50, 19, 10, 4, 15, 2, 63, 8, 97, 9, 9, 16,
                    88, 7, 64, 10, 12, 11, 95, 18, 19, 12, 16,
                    14, 54, 15, 9 },
                { 18, 52, 17, 33, 5, 16, 10, 10, 8, 87, 9, 62, 0,
                    76, 11, 88, 12, 22, 2, 63, 19, 50, 14, 87,
                    16, 16, 1, 56, 6, 38, 15, 57, 4, 52, 3, 71,
                    7, 15, 13, 18 },
                { 8, 89, 15, 32, 1, 77, 6, 66, 2, 88, 19, 6, 16,
                    37, 17, 35, 0, 49, 10, 50, 12, 89, 5, 62, 9,
                    28, 13, 31, 4, 61, 14, 41, 3, 71, 11, 2, 18,
                    22, 7, 9 },
                { 7, 32, 8, 20, 3, 77, 17, 70, 14, 23, 5, 82, 1,
                    51, 11, 75, 9, 34, 15, 55, 4, 91, 12, 49, 16,
                    46, 6, 80, 10, 6, 2, 69, 13, 70, 19, 99, 18,
                    85, 0, 45 },
                { 18, 32, 15, 27, 8, 43, 16, 53, 13, 70, 6, 17,
                    11, 57, 4, 77, 10, 91, 2, 49, 3, 6, 1, 6, 19,
                    47, 7, 93, 12, 4, 9, 54, 0, 93, 14, 68, 17,
                    48, 5, 93 },
                { 11, 16, 3, 5, 10, 39, 17, 79, 19, 51, 16, 39,
                    5, 39, 6, 32, 13, 38, 9, 28, 8, 96, 14, 35,
                    12, 56, 18, 42, 1, 45, 15, 83, 2, 42, 4, 40,
                    7, 25, 0, 3 },
                { 11, 77, 8, 1, 5, 11, 13, 87, 18, 21, 16, 44, 1,
                    92, 3, 77, 7, 9, 6, 92, 9, 10, 14, 31, 10,
                    62, 2, 65, 4, 44, 19, 78, 15, 22, 17, 13, 0,
                    26, 12, 72 },
                { 17, 11, 19, 46, 8, 8, 0, 77, 4, 31, 1, 48, 11,
                    37, 3, 99, 13, 27, 16, 99, 9, 31, 10, 73, 5,
                    60, 14, 40, 2, 81, 18, 9, 12, 20, 7, 81, 15,
                    71, 6, 2 },
                { 6, 60, 5, 20, 1, 62, 18, 18, 7, 18, 0, 33, 19,
                    38, 16, 48, 2, 37, 12, 78, 14, 76, 17, 28, 3,
                    65, 10, 61, 8, 40, 4, 28, 15, 67, 13, 4, 9,
                    17, 11, 26 },
                { 2, 54, 17, 89, 7, 69, 12, 93, 0, 69, 14, 93, 4,
                    54, 19, 30, 15, 14, 10, 14, 8, 71, 1, 49, 6,
                    10, 3, 86, 18, 88, 5, 67, 9, 17, 11, 2, 16,
                    2, 13, 20 },
                { 16, 54, 12, 83, 0, 59, 2, 70, 14, 18, 5, 34, 6,
                    7, 3, 2, 17, 45, 15, 6, 18, 80, 10, 21, 11,
                    72, 7, 80, 9, 89, 1, 52, 8, 33, 19, 41, 4,
                    76, 13, 5 },
                { 9, 53, 3, 15, 11, 75, 18, 10, 15, 48, 1, 54,
                    19, 1, 16, 43, 13, 22, 17, 31, 0, 63, 5, 1,
                    14, 68, 10, 74, 12, 23, 4, 32, 2, 53, 8, 71,
                    6, 93, 7, 22 },
                { 16, 21, 14, 51, 2, 56, 11, 39, 5, 32, 13, 17,
                    17, 37, 19, 21, 8, 42, 15, 81, 9, 56, 10, 29,
                    1, 11, 12, 78, 7, 40, 4, 66, 6, 57, 3, 92,
                    18, 5, 0, 38 },
                { 7, 30, 9, 31, 16, 92, 2, 26, 14, 56, 3, 67, 5,
                    88, 6, 11, 18, 25, 19, 24, 13, 98, 8, 8, 15,
                    34, 17, 18, 0, 80, 12, 82, 11, 28, 10, 87, 4,
                    77, 1, 14 },
                { 5, 40, 18, 43, 13, 85, 10, 84, 8, 76, 14, 64,
                    9, 7, 12, 51, 4, 51, 2, 15, 7, 96, 11, 31,
                    19, 85, 6, 30, 1, 85, 17, 45, 0, 73, 15, 73,
                    3, 17, 16, 57 },
                { 6, 3, 11, 32, 8, 92, 0, 97, 1, 78, 2, 83, 15,
                    37, 7, 41, 12, 4, 14, 62, 18, 16, 5, 36, 16,
                    4, 10, 26, 19, 28, 17, 2, 4, 16, 3, 37, 9,
                    93, 13, 26 },
                { 7, 64, 10, 90, 16, 88, 9, 32, 5, 13, 15, 70,
                    12, 5, 6, 30, 3, 44, 2, 82, 0, 98, 19, 44,
                    18, 67, 8, 24, 11, 49, 4, 99, 17, 9, 1, 28,
                    13, 96, 14, 70 },
                { 15, 83, 18, 29, 19, 27, 0, 78, 10, 85, 12, 10,
                    14, 77, 9, 91, 1, 24, 3, 97, 17, 19, 2, 54,
                    8, 40, 6, 39, 13, 46, 16, 89, 7, 83, 5, 93,
                    11, 53, 4, 90 },
                { 13, 76, 19, 56, 7, 40, 4, 73, 6, 14, 18, 74,
                    12, 75, 1, 55, 17, 90, 16, 8, 2, 9, 14, 59,
                    3, 60, 5, 27, 15, 1, 0, 17, 11, 6, 9, 41, 10,
                    41, 8, 10 },
                { 0, 23, 3, 77, 8, 7, 12, 66, 9, 5, 16, 85, 2,
                    87, 6, 1, 19, 40, 14, 69, 5, 62, 17, 90, 11,
                    20, 15, 18, 4, 32, 1, 36, 18, 4, 7, 6, 10,
                    47, 13, 28 },
                { 11, 45, 9, 87, 19, 4, 3, 17, 1, 6, 7, 13, 2,
                    72, 4, 66, 18, 68, 6, 87, 15, 1, 17, 79, 16,
                    44, 14, 5, 10, 33, 0, 32, 12, 20, 8, 63, 13,
                    73, 5, 62 },
                { 10, 81, 5, 3, 9, 13, 8, 42, 3, 22, 1, 32, 0,
                    83, 4, 56, 7, 28, 15, 96, 11, 34, 19, 42, 18,
                    22, 13, 86, 14, 45, 17, 79, 2, 7, 12, 43, 16,
                    27, 6, 41 },
                { 6, 4, 10, 55, 4, 31, 19, 22, 2, 28, 13, 44, 12,
                    15, 11, 88, 8, 27, 9, 66, 17, 50, 0, 25, 1,
                    27, 7, 91, 14, 53, 15, 71, 3, 10, 16, 12, 5,
                    2, 18, 61 },
                { 11, 70, 9, 69, 8, 26, 16, 78, 13, 42, 7, 97,
                    19, 68, 5, 88, 10, 37, 3, 23, 18, 46, 2, 55,
                    14, 29, 17, 81, 12, 54, 4, 70, 0, 71, 6, 2,
                    1, 53, 15, 28 },
                { 11, 39, 12, 56, 9, 87, 7, 21, 15, 9, 18, 82,
                    16, 44, 8, 63, 19, 9, 4, 50, 1, 34, 17, 58,
                    13, 18, 5, 87, 0, 42, 14, 78, 2, 98, 6, 30,
                    10, 46, 3, 21 } },
                "ta80"))).lowerBound()); //$NON-NLS-1$
  }
}
